
% Minimal example of tracking with MIMU22BT modules

% Clear workspac
clear all, clc, close all

% Open serial ports
com = serial('COM83','ByteOrder','bigEndian');
fopen(com);
% Flush serial ports
pause(1);
while com.BytesAvailable
    fread(com,com.BytesAvailable,'uint8');
end

% Prepare figures and plotting variables
fig1=figure(1);
hold on, grid on, axis equal
title('Real-time position plot'); xlabel('x [m]'); ylabel('y [m]');
set (fig1, 'Units', 'normalized', 'Position', [0,0.05,1,0.95]);
% Add pushbutton such that logging can be aborted
abort_flag = 0;
uicontrol('style','push','string','Stop','callback','abort_flag=1;');
drawnow
pause(0.1)

% State variables
x = zeros(4,1);
P = zeros(4,4);
x_old = x;

% Tell the modules to start
fwrite(com,[52 0 52],'uint8');
fread(com,4,'uint8') % Read ack

% Loop as long as stop-botton is not pressed
step_counter=0;
dP_scale=20;
package_number_old = nan;
while abort_flag==0
    if(com.BytesAvailable>=64)
        %fread(com.BytesAvailable,'uint8')'
        header = fread(com,4,'uint8'); % Header + number (16 bits) + Payload size
        payload = fread(com,14,'float'); % 3 dx,dy,dz, heading, upper triangle of error covariance matrix
        step_counter = fread(com,1,'uint16'); % Step counter
        fread(com,1,'uint16'); % checksum 
        package_number1 = header(2);
        package_number2 = header(3);
        ack = [1 package_number1 package_number2 (1+package_number1+package_number2-mod(1+package_number1+package_number2,256))/256 mod(1+package_number1+package_number2,256)];
        fwrite(com,ack,'uint8');
        package_number = package_number1*256 + package_number2;
        if package_number_old ~= package_number
            dx = payload(1:4);
            dP = dP_scale*Pvec2Pmat(payload(5:14));
            [x,P] = stepwise_dr_tu(x,P,dx,dP);
            plot([x_old(1) x(1)],[x_old(2) x(2)]);
            x_old=x;
            axis equal; drawnow;
            package_number_old = package_number;
            fprintf('Package number: %i, step number: %i\n',package_number_old,step_counter);
        end
    end
end

fwrite(com,[50 0 50],'uint8'); % Stop processing
fwrite(com,[34 0 34],'uint8'); % Stop output (and empty buffers)
pause(1);

while com.BytesAvailable
    fread(com,com.BytesAvailable,'uint8');
end
fclose(com);

